@@ -8,16 +8,30 @@ import threading
 
 
 class WatchPrint:
-    def __init__(self, file=sys.stderr):
+    def __init__(self, file=sys.stderr, stack_limit=None):
         self.file = file
+        self.stack_limit = stack_limit
 
     def __call__(self, frame, elem, exec_info):
         p = self.printer
+        p("====== Watchpoints Triggered ======")
         if threading.active_count() > 1:
             curr_thread = threading.current_thread()
-            p(f"> {curr_thread.name}")
+            p(f"---- {curr_thread.name} ----")
+        p("Call Stack (most recent call last):")
+
+        curr_frame = frame.f_back
+        frame_counter = 0
+        trace_back_data = []
+        while curr_frame and (self.stack_limit is None or frame_counter < self.stack_limit - 1):
+            trace_back_data.append(self._frame_string(curr_frame))
+            curr_frame = curr_frame.f_back
+            frame_counter += 1
+
+        for s in trace_back_data[::-1]:
+            p(s)
+
         p(self._file_string(exec_info))
-        p(f"> {self.getsourceline(exec_info)}")
         if elem.alias:
             p(f"{elem.alias}:")
         elif elem.default_alias:
@@ -28,13 +42,17 @@ class WatchPrint:
         p("")
 
     def _file_string(self, exec_info):
-        return f"> {exec_info[0]} ({exec_info[1]}:{exec_info[2]}):"
+        return f"  {exec_info[0]} ({exec_info[1]}:{exec_info[2]}):\n" + \
+            self.getsourceline(exec_info)
+
+    def _frame_string(self, frame):
+        return self._file_string((frame.f_code.co_name, frame.f_code.co_filename, frame.f_lineno))
 
     def getsourceline(self, exec_info):
         try:
             with open(exec_info[1]) as f:
                 lines = f.readlines()
-                return f"    {lines[exec_info[2] - 1].strip()}"
+                return f">   {lines[exec_info[2] - 1].strip()}"
         except (FileNotFoundError, PermissionError):
             return "unable to locate the source"
 
